/**
 *--------------------------------------------------------------------\n
 *          HSLU T&A Hochschule Luzern Technik+Architektur            \n
 *--------------------------------------------------------------------\n
 *
 * \brief         terminal with command line parser
 * \file
 * \author        Christian Jost, christian.jost@hslu.ch
 * \date          30.11.2012
 *
 * \b Language:   Ansi-C \n\n
 * \b Target:     MC-Car  \n
 *
 * \par Description:
 *
 * $Id: term.c 399 2012-11-30 15:23:13Z zajost $
 *--------------------------------------------------------------------
 */
#include "platform.h"
#include "term.h"
#include "sci.h"
#include <string.h>
#include "util.h"
#include "pid.h"



void termWriteNum16s(int16 value)
{
  char str[8];
  utilNum16sToStr(str, sizeof(str), value);
  sciWrite(str);
}

void termWriteNum16u(uint16 value)
{
  char str[8];
  utilNum16uToStr(str, sizeof(str), value);
  sciWrite(str);
}


void sciWriteSubString(char *str, uint8 start, uint8 length)
{
  str += start;
  while(length-- > 0) 
  {
    sci1WriteChar(*(str++));
    sci2WriteChar(*(str++));
  }
}


void termParseCommand(char *cmd)
{
  uint8 subCmdLength = 0;
  tError result = EC_INVALID_CMD;
  
  if (strcmp(cmd, "help") == 0)
  {
    sciWriteLine(NULL);
    sciWriteLine(">>> MC-Car terminal <<<");
    sciWriteLine("valid commands are:");
    sciWriteLine("  pid help");
    result = EC_SUCCESS;
  }
  else if (strncmp(cmd, "pid ", sizeof("pid")) == 0)
  {
    subCmdLength = sizeof("pid ") - 1;
    result = pidParseCommand(cmd + sizeof("pid"));
  }
  if (result != EC_SUCCESS)
  {
    sciWrite(cmd);
    sciWriteLine(": command not found!");
    sciWrite("Enter '");
    sciWriteSubString(cmd, 0, subCmdLength);
    sciWriteLine("help' for a list of valid commands.");
  }
}


void termDoWork(void)
{
  char cmd[64];
  if  (sciHasLineReceived())
  {
    (void)sciReadLine(cmd, sizeof(cmd));
    termParseCommand(cmd);
  }
}

void termInit(void)
{
  sciWriteLine("MC-Car ready :-)");
}